
// Code from http://www.generative-gestaltung.de

/**
 * more nodes and more springs
 *
 * KEYS
 * r             : reset positions
 * s             : save png
 * p             : save pdf
 */

/* 
 Rserve (rforge.net/Rserve/) is a server which allows other programs to use facilities of R 
 from various languages without the need to initialize R or link against R library.
 
 Use as follows:
 1. On R install package Rserve and then run: 
 library(Rserve); Rserve(args = "--vanilla")
 2. Run the Processing sketch
 3. Stop the Rserve server from console with: 
 killall -9 Rserve 
 */

// import Rserve libraries
import org.rosuda.REngine.Rserve.*;
import org.rosuda.REngine.*;
import processing.pdf.*;
import java.util.Calendar;

boolean savePDF = false;
float nodeDiameter = 16;
// an array for the nodes
Node[] nodes;
// an array for the springs
Spring[] springs;
// dragged node
Node selectedNode = null;

// graph samples

//String graph = "sample_pa(n = 80, m = 1, directed=FALSE)";
//String graph = "sample_gnp(n = 100, p = 0.02, directed=FALSE)";
//String graph = "make_star(n = 20)";
//String graph = "make_tree(n = 50, mode=\"undirected\")";
//String graph = "make_ring(n = 20)";
//String graph = "make_lattice(c(6, 6))";
//String graph = "make_lattice(c(2, 2, 2))";
//String graph = "make_lattice(c(3, 3, 3))";
// String graph = "make_lattice(c(2, 2, 2, 2))";
// String graph = "make_full_graph(n = 10)";
//String graph = "make_graph(edges = \"Zachary\")";
//String graph = "make_graph(edges = \"Dodecahedron\")";
String graph = "sample_k_regular(no.of.nodes=20, k=3)";
String comGenGraph = "library(igraph); g = " + graph + "; n = vcount(g); m = ecount(g); E = ends(g, es=E(g)); x = E[, 1]; y = E[, 2];";

void setup() {
  size(700, 700);
  smooth();
  noStroke();
  initNodesAndSprings();
}


void draw() {
  if (savePDF) beginRecord(PDF, timestamp()+".pdf");

  background(255);
  // with reverberation
  //fill(0, 0, 0, 10);
  //rect(0, 0, width, height);

  // apply repell forces among nodes
  for (int i = 0; i < nodes.length; i++) {
    nodes[i].attract(nodes);
  } 

  // apply spring forces among nodes
  for (int i = 0; i < springs.length; i++) {
    springs[i].update();
  } 

  // update positions of nodes
  for (int i = 0; i < nodes.length; i++) {
    nodes[i].update();
  } 

  // draw springs
  for (int i = 0; i < springs.length; i++) {
    springs[i].show();
  }

  // draw nodes
  for (int i = 0; i < nodes.length; i++) {
    nodes[i].show(nodeDiameter);
  }

  if (selectedNode != null) {
    selectedNode.x = mouseX;
    selectedNode.y = mouseY;
  }

  if (savePDF) {
    savePDF = false;
    println("saving to pdf – finishing");
    endRecord();
  }
}


void initNodesAndSprings() {
  int n = 0, m = 0;
  int[] x;
  int[] y;

  try {
    // start connection
    RConnection c = new RConnection();  

    // generate random graph
    c.eval(comGenGraph);

    // get number of nodes
    n = c.eval("n").asInteger();
    //println(n);

    // get number of edges
    m = c.eval("m").asInteger();
    //println(m);


    // get edges
    x = new int[n];
    y = new int[n];
    x = c.eval("x").asIntegers();
    y = c.eval("y").asIntegers();
    //println(x);
    //println(y);

    // init nodes
    nodes = new Node[n];
    float rad = nodeDiameter/2;
    for (int i = 0; i < nodes.length; i++) {
      nodes[i] = new Node(width/2+random(-200, 200), height/2+random(-200, 200));
      nodes[i].setBoundary(rad, rad, width-rad, height-rad);
      nodes[i].setRadius(100);
      nodes[i].setStrength(-5);
    } 

    // set springs
    springs = new Spring[m];
    for (int i = 0; i < m; i++) {
      Spring newSpring = new Spring(nodes[x[i] - 1], nodes[y[i] - 1]);
      // force edges with the same length
      newSpring.setLength(20);
      // speed of convergenge to the final layout
      newSpring.setStiffness(1);
      springs[i] = newSpring;
    }


    // close connection
    c.close();
  } 
  catch ( REXPMismatchException rme ) {
    rme.printStackTrace();
  } 
  catch ( REngineException ree ) {
    ree.printStackTrace();
  }
}


void mousePressed() {
  // Ignore anything greater than this distance
  float maxDist = 20;
  for (int i = 0; i < nodes.length; i++) {
    Node checkNode = nodes[i];
    float d = dist(mouseX, mouseY, checkNode.x, checkNode.y);
    if (d < maxDist) {
      selectedNode = checkNode;
      maxDist = d;
    }
  }
}

void mouseReleased() {
  if (selectedNode != null) {
    selectedNode = null;
  }
}


void keyPressed() {
  if (key=='s' || key=='S') saveFrame(timestamp()+"_##.png"); 

  if (key=='p' || key=='P') {
    savePDF = true; 
    println("saving to pdf - starting (this may take some time)");
  }

  if (key=='r' || key=='R') {
    background(255);
    initNodesAndSprings();
  }
}


String timestamp() {
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", Calendar.getInstance());
}